/*---------------------------------------------------------------------------*/
/* HTML-Patcher																 */
/*---------------------------------------------------------------------------*/
/* Programm zum patchen von HTML-Seiten										 */
/*---------------------------------------------------------------------------*/
/* Version																	 */
/* 1.0 21.11.1997 Ralf Seyfried												 */
/* 2.0 21.11.1997 Axel Mueller												 */
/* 3.0 09.01.2002 Axel Mller												 */
/*---------------------------------------------------------------------------*/

// includes

#include "stdafx.h"

#include <string.h>
#include <stdio.h>
#include <fcntl.h>
#include <io.h>

#include <time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/utime.h>

// defines
#define UPDATE_FILE_DATE

// constants
#define MAX_TOKEN_LEN	100

#define TM_COMMENT 		1
#define TM_NORMAL		2
#define RM_REPLACEFILE	1
#define RM_REPLACETEXT	2

#define FILE_DATE_TAG 		"FILEDATE"
#define FILE_TIME_TAG 		"FILETIME"
#define ORIGINAL_TEXT_TAG	"<!--ORIGINALTEXT-->"
#define BEFORE_TOKEN		1
#define BETWEEN_TOKEN		2
#define INSERT_TEXT			3

#define TEMP_FILE_NAME	"tempfile.$$$"

// variables
char LineBuffer[1000];
char LineBuffer2[1000];
char LineBuffer3[1000];


// functions

int FileExists (char * filename)
{
	FILE *f;
	f = fopen (filename, "r+");
	if (f == NULL)
		return (0);
	fclose (f);
	return (1);
}

int ReadLine (FILE *file, char *line_buf)
{
	char ch;
	int pos;

	printf (".");
	pos = 0;
	ch = fgetc (file);
	if (ch == EOF) return (0);
	while ((ch != EOF) && (ch != 10))
	{
		line_buf[pos++] = ch;
		ch = fgetc (file);
	}
	if (ch == 10) line_buf[pos++] = 10;
	line_buf[pos] = '\0';
	return (1);
}

void BuildTokens (char *text, char *start, char *stop, int tagMode)
{
	if (tagMode == TM_COMMENT)
	{
		strcpy (start, "<!--");
		strcat (start, text);
		strcat (start, "-->");
		strcpy (stop, "<!--/");
		strcat (stop, text);
		strcat (stop, "-->");
	}
	else if (tagMode == TM_NORMAL)
	{
		strcpy (start, "<");
		strcat (start, text);
		strcat (start, ">");
		strcpy (stop, "</");
		strcat (stop, text);
		strcat (stop, ">");
	}
}

void ReplaceTextBetweenTokens (char *source_file_name,
							   char *token,
							   char *text,
							   int tagMode,
							   int replaceMode)
{
	FILE *sourcefile;
	FILE *destfile;
	FILE *tempfile;
	FILE *replacefile;
	char BackupFileName[100];
	char ch;
	int pos;
	char StartToken[MAX_TOKEN_LEN];
	char StopToken[MAX_TOKEN_LEN];
	int mode;
	char *start;
	char *stop;
	char *poschar;
	char *tmppos;
	int ok;

	// make a backup
	pos = 0;
	do
	{
		ch = source_file_name[pos];
		BackupFileName[pos] = ch;
		pos++;
	} while ((ch != '.') && (ch != '\0'));
	BackupFileName[pos] = '\0';
	if (ch == '\0') BackupFileName[pos++] = '.';
	BackupFileName[pos++] = 'b';
	BackupFileName[pos++] = 'a';
	BackupFileName[pos++] = 'k';
	BackupFileName[pos] = '\0';

	remove (BackupFileName);
	rename (source_file_name, BackupFileName);

	BuildTokens (strupr(token), StartToken, StopToken, tagMode);
	sourcefile = fopen (BackupFileName, "r");
	destfile   = fopen (source_file_name, "w+");
	mode = BEFORE_TOKEN;
	ok = ReadLine (sourcefile, LineBuffer);
	while (ok)
	{
		// printf ("m:%u -^%s^-\n", mode, LineBuffer);
		switch (mode)
		{
			case BEFORE_TOKEN:
				start = strstr (LineBuffer, StartToken);
				if (start == NULL)
				{
					// Token not found
					fputs (LineBuffer, destfile);
					ok = ReadLine (sourcefile, LineBuffer);
					start = LineBuffer;
				}
				else
				{
					// Token found
					poschar = LineBuffer;
					while (poschar != start)
					{
						fputc (*poschar, destfile);
						poschar++;
					}
					fputs (StartToken, destfile);
					start += strlen(StartToken);
					if (replaceMode == RM_REPLACEFILE)
						tempfile = fopen (TEMP_FILE_NAME, "w+");
					mode = BETWEEN_TOKEN;
				}
				break;
			case BETWEEN_TOKEN:
				stop = strstr (start, StopToken);
				if (stop == NULL)
				{
					// Zeile kann in Zwischendatei geschrieben werden
					if (replaceMode == RM_REPLACEFILE)
						fputs (start, tempfile);
					ok = ReadLine (sourcefile, LineBuffer);
					start = LineBuffer;
				}
				else
				{
					// Bis zum Token schreiben
					poschar = start;
					while (poschar != stop)
					{
						if (replaceMode == RM_REPLACEFILE)
							fputc (*poschar, tempfile);
						poschar++;
					}
					stop += strlen(StopToken);
					mode = INSERT_TEXT;
				}
				break;
			case INSERT_TEXT:
				if (replaceMode == RM_REPLACEFILE)
				{
					rewind (tempfile);
					replacefile = fopen (text, "r");
					while (ReadLine (replacefile, LineBuffer2))
					{
						tmppos = strstr (LineBuffer2, ORIGINAL_TEXT_TAG);
						if (tmppos == NULL)
						{
							fputs (LineBuffer2, destfile);
						}
						else
						{
							poschar = LineBuffer2;
							while (poschar != tmppos)
							{
								fputc (*poschar, destfile);
								poschar++;
							}
							while (ReadLine (tempfile, LineBuffer3))
								fputs (LineBuffer3, destfile);
							tmppos += strlen (ORIGINAL_TEXT_TAG);
							fputs (tmppos, destfile);
						}
					}
					fclose (replacefile);
					fclose (tempfile);
					remove (TEMP_FILE_NAME);
				}
				else
				{
					fputs (text, destfile);
				}
				fputs (StopToken, destfile);
				fputs (stop, destfile);
				ok = ReadLine (sourcefile, LineBuffer);
				mode = BEFORE_TOKEN;
				break;
		}
	}
	fclose (destfile);
	fclose (sourcefile);
}


int ReadFileTime (char *FileName, time_t *ft)
{
	struct _stat buffer;

	_stat (FileName, &buffer);
	*ft = buffer.st_mtime;

/*

	file = fopen (FileName, "r");
	if (file == NULL) return (0);
	getftime (fileno(file), ft);
	fclose (file);

*/
	return (1);
}

void WriteFileTime (char *FileName, time_t *ft)
{
#ifdef UPDATE_FILE_DATE
	struct _utimbuf buffer;

	buffer.actime = *ft;
	buffer.modtime = *ft;

	_utime (FileName, &buffer);
/*
	FILE *file;

	file = fopen (FileName, "r+");
	setftime (fileno(file), ft);
	fclose (file);
*/
#endif
}

// main

int main(int argc, char * argv[])
{
	int mode;
	int tagMode;
	time_t ft;
//	char filedate[100];
	char SourceFileName[100];
//	char DestFileName[100];
	char ReplaceFileName[100];
	char ReplaceText[100];
	char Tag[100];
	int i;

	if ((argc <= 2) || (strcmp (argv[1], "/?") == 0))
	{
		printf ("HTMLPATCH - Patcher fuer HTML-Dateien\n\n");
		printf ("Aufruf 1: HTMLPATCH.EXE /r <Sourcefile> <Tag> <Replacefile>\n");
		printf ("          Ersetzen des Textes zwischen zwei Kommentaren\n");
		printf ("          <!--Tag--> und <!--/Tag--> in der Sourcedatei\n");
		printf ("          durch den Inhalt von Replacefile\n");
		printf ("Aufruf 2: HTMLPATCH.EXE /t <Sourcefile> <Tag> <Text>\n");
		printf ("          Ersetzen des Textes zwischen zwei Kommentaren\n");
		printf ("          <!--Tag--> und <!--/Tag--> durch den Text <Text>\n");
		printf ("Aufruf 3: HTMLPATCH.EXE /fd <Sourcefile>\n");
		printf ("          Ersetzen des Textes zwischen zwei Kommentaren\n");
		printf ("          <!--%s--> und <!--/%s--> durch das\n", FILE_DATE_TAG, FILE_DATE_TAG);
		printf ("          aktuelle Dateidatum\n");
		printf ("Aufruf 4: HTMLPATCH.EXE /ft <Sourcefile>\n");
		printf ("          Ersetzen des Textes zwischen zwei Kommentaren\n");
		printf ("          <!--%s--> und <!--/%s--> durch die\n", FILE_TIME_TAG, FILE_TIME_TAG);
		printf ("          aktuelle Dateizeit\n");
		printf ("Statt /r bzw. /t kann /nr bzw. /nt angegeben werden, damit werden\n");
		printf ("dann keine Kommentare, sondern HTML-Tags gesucht und ersetzt.\n");
		printf ("Mit dem Token <!--ORIGINALTEXT--> im Replacefile kann der Originaltext\n");		
		printf ("bei der Ersetzung mit eingebunden werden.\n");
		return (1);
	} 
	
	if      (strcmp (argv[1], "/t")  == 0) 
	{
		mode = 2;
		tagMode = TM_COMMENT;
	}
	else if (strcmp (argv[1], "/nt") == 0)
	{
		mode = 2;
		tagMode = TM_NORMAL;
	}
	else if (strcmp (argv[1], "/fd") == 0)
	{
		mode = 3;
		tagMode = TM_COMMENT;
	}
	else if (strcmp (argv[1], "/ft") == 0)
	{
		mode = 4;
		tagMode = TM_COMMENT;
	}
	else if (strcmp (argv[1], "/nr") == 0)
	{
		mode = 1;
		tagMode = TM_NORMAL;
	}
	else if (strcmp (argv[1], "/r") == 0)
	{
		mode = 1;
		tagMode = TM_COMMENT;
	}
	else
	{
		printf ("Bitte geben Sie ein Kommando mit /... an.\n\n");
		return (1);
	}

	if (!FileExists(argv[2]))
	{
		printf ("Die Datei %s existiert nicht !\n\n", argv[2]);
		return (1);
	}


	switch (mode)
	{
		case 1:
			if (argc != 5)
			{
				printf ("Bitte geben Sie 4 Parameter an\n");
				return (1);
			} 
			strcpy (SourceFileName, argv[2]);
			strcpy (Tag, argv[3]);
			strcpy (ReplaceFileName, argv[4]);
			if (!FileExists (ReplaceFileName))
			{
				printf ("Die Datei %s existiert nicht !\n\n", ReplaceFileName);
				return (1);
			}
			ReplaceTextBetweenTokens (SourceFileName, 
										Tag, 
										ReplaceFileName, 
										tagMode, 
										RM_REPLACEFILE);
			break;
		case 2:
			if (argc < 5)
			{
				printf ("Bitte geben Sie mindestens 3 Parameter an\n");
				return (1);
			} 
			strcpy (SourceFileName, argv[2]);
			strcpy (Tag, argv[3]);
			strcpy (ReplaceText, argv[4]);
			for (i=5; i<argc; i++)
			{
				strcat (ReplaceText, " ");
				strcat (ReplaceText, argv[i]);
			}
			ReplaceTextBetweenTokens (SourceFileName, 
										Tag, 
										ReplaceText, 
										tagMode, 
										RM_REPLACETEXT);
			break;
		case 3:
			if (argc != 3)
			{
				printf ("Bitte geben Sie 1 Parameter an\n");
				return (1);
			} 
			strcpy (SourceFileName, argv[2]);
			if (!ReadFileTime (SourceFileName, &ft))
			{
				printf ("File %s nicht gefunden !\n", SourceFileName);
				return (1);
			}

			strftime (ReplaceText, 100, "%Y-%m-%d", gmtime (&ft));
			// sprintf (ReplaceText, "%2u.%2u.%4u", ft.ft_day, ft.ft_month, ft.ft_year+1980);
			ReplaceTextBetweenTokens (SourceFileName, 
										FILE_DATE_TAG, 
										ReplaceText, 
										tagMode, 
										RM_REPLACETEXT);
			WriteFileTime (SourceFileName, &ft);
			break;
		case 4:
			if (argc != 3)
			{
				printf ("Bitte geben Sie 1 Parameter an\n");
				return (1);
			} 
			strcpy (SourceFileName, argv[2]);
			if (!ReadFileTime (SourceFileName, &ft))
			{
				printf ("File %s nicht gefunden !\n", SourceFileName);
				return (1);
			}
			strftime (ReplaceText, 100, "%H:%M:%S", gmtime (&ft));
			// sprintf (ReplaceText, "%2u:%2u:%2u", ft.ft_hour, ft.ft_min, ft.ft_tsec*2);
			ReplaceTextBetweenTokens (SourceFileName, 
										FILE_TIME_TAG, 
										ReplaceText, 
										tagMode,
										RM_REPLACETEXT);
			WriteFileTime (SourceFileName, &ft);
			break;
	}
	return 0;
}

